
var child_process = require('child_process')
  , path = require('path')
  , temp = require('temp')
  , async = require('async')
  , mkdirp = require('mkdirp')
  , fs = require('fs')
  , GEF = exports;


GEF.getPreviousRevisionCommitID = function (fullPath, callback)
{
    var opts = {
        cwd: path.dirname( fullPath )
    };

    child_process.exec('git log -1 --format=format:%H "' + fullPath + '"', opts, function (err, stdout, stderr) {
        if (err) { return callback( err ); }
        if (stderr !== null && stderr.toString().length > 0) { return callback( stderr ); }

        var previousRevisionCommit = stdout.toString().trim();

        return callback( null, previousRevisionCommit );
    });
};

GEF.getContentsOfFileAtRevision = function (fullPath, revision, callback)
{
    var opts = {
        cwd: path.dirname( fullPath )
    };

    var filename = path.basename( fullPath );
    var cmd = 'git show "' + revision + ':./' + filename + '"'

    child_process.exec(cmd, opts, function (err, stdout, stderr) {
        if (err) { return callback( err ); }
        if (stderr !== null && stderr.toString().length > 0) { return callback( stderr ); }

        return callback( null, stdout.toString() )
    });

};


GEF.ensureDestinationDir = function (destDir, callback)
{
    if (destDir)
    {
        mkdirp(destDir, function (err) {
            return callback(err, destDir);
        });
    }

    // if destDir is null, we create a temp dir automatically
    else
    {
        var tempDirPath = temp.path({ prefix: 'git-export-file--' });

        mkdirp(tempDirPath, function (err) {
            return callback(err, tempDirPath);
        });
    }
};


GEF.ensureRevision = function (fullPath, revision, callback)
{
    if (revision)
    {
        return callback(null, revision);
    }
    else
    {
        GEF.getPreviousRevisionCommitID(fullPath, function (err, previousRevisionCommit) {
            return callback(err, previousRevisionCommit);
        })
    }
}

GEF.exportRevisionOfDirToDir = function (fullPath, revision, dstDir, callback)
{
    var minimatch = require('minimatch')
      , walk = require('walk')
      , dir_fullPath = path.resolve( fullPath );

    GEF.ensureDestinationDir(dstDir, function (err, dstDir) {

        var ignorePatterns = ['.git', '.idea', 'Pods/*', '*.xcworkspace', '*.xcodeproj', '.DS_Store', '*.strings', '*.sublime-workspace'];

        var walker = walk.walk( dir_fullPath, { followLinks: false , filters: ['.git'] } ); // 'filters' doesn't seem to work for fuck, hence the manual implementation below

        function matchesIgnorePattern(root, itemName) {
            var item_fullPath     = path.join( root, itemName );
            var item_relativePath = path.relative( dir_fullPath, item_fullPath );

            for ( var p = 0; p < ignorePatterns.length; p++ ) {
                if (minimatch( item_relativePath, ignorePatterns[p] ) || minimatch( itemName, ignorePatterns[p] )) {
                    return true;
                }
            }
            return false;
        };

        walker.on('directories', function (root, dirStatsArray, next) {
            for ( var i = dirStatsArray.length - 1; i >= 0; i-- )
            {
                var currentDir_fullPath     = path.join( root, dirStatsArray[i].name );
                var currentDir_relativePath = path.relative( dir_fullPath, currentDir_fullPath );

                if (matchesIgnorePattern( root, dirStatsArray[i].name )) {
                    dirStatsArray.splice(i, 1);
                }
            }

            next();
        });

        walker.on('file', function (root, fileStats, next) {
            var file_fullPath     = path.join( root, fileStats.name );
            var file_relativePath = path.relative( dir_fullPath, file_fullPath );

            if (matchesIgnorePattern( root, fileStats.name )) {
                // no-op
                next();
            }
            else {
                var file_relativeDestDir = path.dirname( file_relativePath );
                var file_fullDestDir = path.resolve( path.join( dstDir, file_relativeDestDir ) );

                GEF.exportRevisionOfFileToDir(file_fullPath, revision, file_fullDestDir, function (err, exportPath) {
                    if (err) {
                        // no-op for now...
                        // return callback( err );
                    }
                    next();
                });
            }

        });

        walker.on('errors', function (root, nodeStatsArray, next) { next(); });
        walker.on('end', function () {
            return callback( null, dstDir );
        });
    });
};

GEF.exportRevisionOfFileToDir = function (fullPath, revision, dstDir, callback)
{
    var filename = path.basename( fullPath );

    async.auto({
        ensure_dest_dir: function (taskCallback) {
            GEF.ensureDestinationDir( dstDir, taskCallback );
        },

        ensure_revision: function (taskCallback) {
            GEF.ensureRevision( fullPath, revision, taskCallback );
        },

        get_file_at_rev: ['ensure_revision', function (taskCallback, results) {
            var revisionToExport = results['ensure_revision'];
            GEF.getContentsOfFileAtRevision( fullPath, revisionToExport, taskCallback );
        }],

        write_file: ['ensure_dest_dir', 'get_file_at_rev', function (taskCallback, results) {
            var fileContents   = results['get_file_at_rev'];
            var destDir        = results['ensure_dest_dir'];
            var exportFilePath = path.join( destDir, filename );

            fs.writeFile(exportFilePath, fileContents, function (err) {
                taskCallback( err, exportFilePath );
            });
        }]
    }, function (err, results) {
        return callback( err, results['write_file'] );
    });
};












